/*---------------------------------------------------------------------------
 * File Name: t3rts_sourcetracking.h
 *
 * Copyright 2001-2004 Telelogic AB, All rights reserved.
 *
 * This Program is owned by Telelogic and is protected by national
 * copyright laws and international copyright treaties. Telelogic
 * grants you the right to use this Program on one computer or in one
 * local computer network at any one time.  Under this License you may
 * only modify the source code for the purpose of adapting it to your
 * environment. You must reproduce and include any copyright and
 * trademark notices on all copies of the source code.  You may not
 * use, copy, merge, modify or transfer the Program except as provided
 * in this License.  Telelogic does not warrant that the Program will
 * meet your requirements or that the operation of the Program will be
 * uninterrupted and error free. You are solely responsible that the
 * selection of the Program and the modification of the source code
 * will achieve your intended results and that the results are
 * actually obtained.
 */

#ifndef __t3rts_sourcetracking_h
#define __t3rts_sourcetracking_h

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>

#include "t3rts_common.h"
#include "t3rts_values_general.h"
#include "t3rts_binarystrings.h"

typedef enum t3rt_scope_kind_internal_t {
  t3rt_scope_function_c,
  t3rt_scope_external_function_c,
  t3rt_scope_testcase_c,
  t3rt_scope_teststep_c,
  t3rt_scope_control_part_c,
  t3rt_scope_undefined_c
} t3rt_scope_kind_t;


typedef struct t3rt__source_location_struct_internal_t *     t3rt_source_location_t;


/* This function retrieves the location on the top of the source tracking
 * stack independent of the the type of the top source location object.
 * See also 't3rt_source_location_get' and  't3rt_targetcode_location_get'.
 */
t3rt_source_location_t
t3rt_source_tracking_top(t3rt_context_t ctx);


/****************************************************************************
 *
 *  TTCN-3 source code location API
 */

/* Pushes a new TTCN-3-related location object on the source tracking stack.
 * The scope argument vector may be NULL.
 */

void
t3rt_source_location_push(const char   *module_name,
                          const char   *scope_name,
                          t3rt_value_t  scope_arguments[],
                          t3rt_scope_kind_t scope_kind,
                          const char   *file_name,
                          unsigned long line_number,
                          t3rt_context_t ctx);

/* Removes top element of the tracking stack.
 */
void
t3rt_source_location_pop(t3rt_context_t ctx);


/* Modifies the current location object with new line information. */
void
t3rt_source_location_set_line(unsigned long line, t3rt_context_t ctx);

/* Returns location created by the last call to 't3rt_source_location_push'. */
t3rt_source_location_t
t3rt_source_location_get(t3rt_context_t ctx);


/* Pushes new TTCN-3 related block scope location */
void
t3rt_source_location_push_block(t3rt_context_t ctx);

/* Pops TTCN-3 related block scope location */
void
t3rt_source_location_pop_block(t3rt_context_t ctx);

/* Registers reference to local (or block) variable */
void
t3rt_source_tracking_register_value(t3rt_value_t value, t3rt_boolean_t rts_owns_value, t3rt_context_t ctx);

/* Locates value reference by the provided name */
t3rt_value_t
t3rt_source_tracking_find_value(const char *value_name, t3rt_context_t ctx);


/****************************************************************************
 *
 *  Target source code location API
 */

/* Pushes a new target code related location object on the targetcode
 * tracking stack.  The associated module and argument vector will be NULL.
 */


void
t3rt_targetcode_location_push(const char *scope_name,
                              const char *file_name,
                              unsigned long line_number,
                              t3rt_context_t ctx);

/* Removes top element of the tracking stack.
 */
void
t3rt_targetcode_location_pop(t3rt_context_t ctx);


/* Modifies the current location object with new line information. */
void
t3rt_targetcode_location_set_line(unsigned long line, t3rt_context_t ctx);

/* Returns location created by the last call to
 * 't3rt_targetcode_location_push'. */
t3rt_source_location_t
t3rt_targetcode_location_get(t3rt_context_t ctx);


/****************************************************************************
 *
 *  Source location object API
 */

/* Returns location created by the last call to 't3rt_source_location_push'
 * (this is an alias for the t3rt_source_location_get function).
 */
t3rt_source_location_t
t3rt_get_source_location(t3rt_context_t ctx);

/* Returns the module name of the source location. */
const char*
t3rt_source_location_module_name(t3rt_source_location_t location,
                                 t3rt_context_t ctx);

/* Returns the scope name of the source location.  */
const char*
t3rt_source_location_scope_name(t3rt_source_location_t location,
                                t3rt_context_t ctx);

/* Returns the scope arguments vector of the source location.  */
t3rt_value_t*
t3rt_source_location_scope_arguments(t3rt_source_location_t location,
                                     t3rt_context_t ctx);

/* Returns the scope kind of the source location.  */
t3rt_scope_kind_t
t3rt_source_location_scope_kind(t3rt_source_location_t location,
                                t3rt_context_t ctx);

/* Returns the file name of the source location.  */
const char*
t3rt_source_location_file_name(t3rt_source_location_t location,
                               t3rt_context_t ctx);

/* Returns the file line of the source location.  */
unsigned long
t3rt_source_location_file_line(t3rt_source_location_t location,
                               t3rt_context_t ctx);

/* Checks whether the location is TTCN-3 or target code related.  */
t3rt_boolean_t
t3rt_source_location_is_ttcn3(t3rt_source_location_t location,
                              t3rt_context_t ctx);



#ifdef __cplusplus
}
#endif

#endif

/* End of File */

